// Frame.cpp: implementation of the CFrame class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "WinClient.h"
#include "Frame.h"
#include <math.h>
#include "RGB2HSV.h"
#include  "basicdef.h"
#include  "filefmtdef.h"
#include  "mcldef.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define MImage_W 352
#define MImage_H 240

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
CFrame::CFrame()
{
	Y=U=V=0;
	CHcalculated = false;
}

CFrame::~CFrame()
{
	int i;

	if (Y&&U&&V)
	{
		for (i=0; i< height; i++)
			free(Y[i]);
		free(Y);

		for (i=0; i< height/2; i++)
		{
			free(U[i]);
			free(V[i]);
		}
		free (U);
		free (V);
	}
	Y=U=V=0;
}
//-------------------------------------------------------------------
int CFrame::loadFrame(CString fileName)
{
	// load the frame from the YUV file previously created this is altered by the function
	// loadFrameFromJPG(fileName)
	WORD wtemp;
	BYTE *btemp;
	int i,j;

	// load the Frame from the file
	FILE *pfFile;
	pfFile=fopen(fileName,"rb");
	if (pfFile==NULL)
	{
		AfxMessageBox("Can't open YUV File.. Work with JPG File Directly");
		return 0;
	}

	// set the width and height
	fread(&wtemp,sizeof(WORD),1,pfFile);
	width = wtemp;
	fread(&wtemp,sizeof(WORD),1,pfFile);
	height = wtemp;

	btemp = new BYTE[width*height];
	
	// Create the YUV arrays
	Y = (BYTE **) malloc(sizeof(BYTE*) * height);
	for (i=0; i < height; i++)
	{
		Y[i] = (BYTE*) malloc(sizeof(BYTE) * width);
	}
	U = (BYTE **) malloc(sizeof(BYTE*) * height/2);
	V = (BYTE **) malloc(sizeof(BYTE*) * height/2);
	for (i=0; i < height/2 ; i++)
	{
		U[i] = (BYTE*) malloc(sizeof(BYTE) * width/2);
		V[i] = (BYTE*) malloc(sizeof(BYTE) * width/2);
	}

	// fill in the YUV arrays
	fread(btemp,sizeof(BYTE),width*height,pfFile);
	for (i=0; i	< height; i++)
		for (j=0 ; j< width; j++)
		{
			Y[i][j] = btemp[i*width + j];
		}

	fread(btemp,sizeof(BYTE),width*height/4,pfFile);
	for (i=0; i	< height/2; i++)
		for (j=0 ; j< width/2; j++)
		{
			U[i][j] = btemp[i*width/2 + j];
		}

	fread(btemp,sizeof(BYTE),width*height/4,pfFile);
	for (i=0; i	< height/2; i++)
		for (j=0 ; j< width/2; j++)
		{
			V[i][j] = btemp[i*width/2 + j];
		}

	delete (btemp);
	fclose(pfFile);
	return 1;
}
//--------------------------------------------------------------------
char * CFrame::getCH(float *h)
{
	char l[1000];
	char temp[20];

	if (!CHcalculated)
		calculateCH();

	memcpy(h,CH,COLQUANT*4);
	strcpy(l,getStringFromArray(CH,COLQUANT));
	strcat(l,"\n");

	return l;
}
//------------------------------------------------------------------------
char * CFrame::getCM(float *h)
{
	float m, v;
	int M = height * width;
	char l[300];
	int i,j;

	m = v = 0;
	for (i=0; i< height; i++)
	{	
		for (j=0;j < width; j++)
		{
			m += Y[i][j];
		}
	}
	
	m = m / M;
	for (i=0; i< height; i++)
	{
		for (j=0; j < width; j++)
		{
			v += getAbsolute(Y[i][j] - m);
		}
	}
	v = v / M;

	h[0] = m;
	h[1] = v;
	strcpy(l,getStringFromArray(h,CM_DIM));
	strcat(l,"\n");
	return l;
}
//---------------------------------------------------------------------------
char * CFrame::getTT(float *h)
{
	const int NOOFBLOCKS=NOOFHBLOCKS*NOOFVBLOCKS;

	float TTA[NOOFBLOCKS], TTcont[NOOFBLOCKS];
	int i,j,b;
	char l[5000], temp[40];

	float M= (float) ((height * width) / NOOFBLOCKS);
	memset(TTA,0,NOOFBLOCKS*4);
	memset(TTcont,0,NOOFBLOCKS*4);

	// calculate color TTA for each block in the frame
	for(i=0;i<height;i++)
	{
		for	(j=0; j< width;j++)
		{
			b = (int) (i*4/height)*4 + (int) (j*4/width);
			TTA[b] += Y[i][j];
		}
	}

	for (i=0; i<NOOFBLOCKS; i++)
		TTA[i] = TTA[i]/(float)M;

	for (i=0;i<height;i++)
	{
		for (j=0;j<width;j++)
		{
			b = (int) (i*4/height)*4 + (int) (j*4/width);
			TTcont[b] += (float) ((Y[i][j]-TTA[b]) * (Y[i][j]-TTA[b]));
		}
	}

	strcpy(l,"");
	for (i=0; i< NOOFBLOCKS; i++)
	{
		TTcont[i] = TTcont[i]/(float) M;
		h[i*2] = TTA[i];
		h[i*2+1] = TTcont[i];

		if (!TTA[i] && !TTcont[i])
			strcat(l," 0 0");
		else 
		{
			if (!TTA[i])
				sprintf(temp," 0 %f",TTcont[i]);
			else if (!TTcont[i])
				sprintf(temp," %f 0",TTA[i]);
			else
				sprintf(temp," %f %f",TTA[i],TTcont[i]);
			strcat(l,temp);
		}
	}
	strcat(l,"\n");
	return l;
}
//-----------------------------------------------------------------------------
char * CFrame::getTE(float *h)
{
	int HOE,VOE,NOE,SOE,T;// values for each pixel
	int f[256], YE[MImage_H][MImage_W], directions[MImage_H][MImage_W];		// array for YE frequencies
	int  i,j;
	float fDirection[TE_DIM];
	char l[2000];

	memset(f,0,256*4);
	memset(fDirection,0,TE_DIM*4);

	// consider only insider pixels
	for (i=1; i<height-1; i++)
	{
		for (j=1; j<width-1; j++)
		{
			// Insider pixel
			HOE = (Y[i-1][j-1] + 2 * Y[i-1][j] + Y[i-1][j+1] - (Y[i+1][j-1] + 2*Y[i+1][j] + Y[i+1][j+1]));
			YE[i][j] = getAbsolute(HOE);
			if (HOE > 0)
				directions[i][j] = 1;
			else
				directions[i][j] = 2;
	
			VOE = (Y[i-1][j-1] + 2*Y[i][j-1] + Y[i+1][j-1] - (Y[i-1][j+1] + 2*Y[i][j+1] + Y[i+1][j+1]));
			if (getAbsolute(VOE) > YE[i][j])
			{
				YE[i][j] = getAbsolute(VOE);
				if (VOE > 0)
					directions[i][j] = 3;
				else
					directions[i][j] = 4;
			}
	
			NOE = (2*Y[i-1][j-1] + Y[i-1][j] + Y[i][j-1] - (Y[i+1][j] + Y[i][j+1] + 2*Y[i+1][j+1]));
			if (getAbsolute(NOE) > YE[i][j])
			{
				YE[i][j] = getAbsolute(NOE);
				if (NOE > 0) 
					directions[i][j] = 5;
				else
					directions[i][j] = 6;
			}
	
			SOE = (Y[i-1][j] + 2*Y[i-1][j+1] + Y[i][j+1] - (Y[i][j-1] + 2*Y[i+1][j-1] + Y[i+1][j]));
			if (getAbsolute(SOE) > YE[i][j])
			{
				YE[i][j] = getAbsolute(SOE);
				if (SOE > 0)
					directions[i][j] = 7;
				else
					directions[i][j] = 8;
			}

			if (YE[i][j] > 255)
				f[255] ++;
			else
				f[YE[i][j]]++;
		}
	}

	// all the frame pixels are processed
	T = getThreshold(f,256,(height * width)-1);

	for (i=1; i<height - 1; i ++)
	{
		for (j=1; j<width - 1; j++)
		{
			if (YE[i][j] >= T)
			{
				fDirection[directions[i][j]]++;
			}
			else
			{
				fDirection[0]++;
			}	
		}
	}

	float M = (height - 2) * (width - 2);
	
	for (i=0; i< TE_DIM; i++)
		h[i] = (float) fDirection[i]/M;

	strcpy(l,getStringFromArray(h,TE_DIM));
	strcat(l,"\n");
	return l;
}
//-----------------------------------------------------------------------

char * CFrame::getCHHSV(float *h)
{
	char l[10000];
	int dim;
	const int M = MImage_W * MImage_H;
	unsigned char r[M], g[M], b[M];
	float *h2;

	// convert YUV2R_G_B
	YUV2R_G_B(this,r,g,b);
	RGB2HSV *rgb2hsv = new RGB2HSV();

	// get the color histogram from HSV format
	h2 = rgb2hsv->R_G_B2HSVHistogram(r,g,b,height * width,&dim);
	if (dim != HSV_DIM)
		AfxMessageBox("error in HSV CH dimension");

	memcpy(h,h2,HSV_DIM*4);
	strcpy(l,getStringFromArray(h,HSV_DIM));
	strcat(l,"\n");

	delete (rgb2hsv);
	return l;
}
//-------------------------------------------------------------------
int CFrame::loadFrameFromJPG(CString fileName)
{
	// this use the MCL library to convert JPG tp YUV
	QmclImg	mImg;	// image handle
	Qint resX;		// image width in pixels
	Qint resY;		// image height in pixels
	Qint *bitsPix;	// image color depth in bits (8, 15, 16, 24).
	Qint *totalBytes;	// total size in bytes occupied by image data (not including the palette). 
	void **pixPtr;	// pointer to the image pixel data
	void **palPtr;	// pointer to the image palette. This information is valid only for 8-bit images.
	Qint r,g,b;		//r,g,b values of a certain pixel
	int i,j;
	char f[100];

	strcpy(f,fileName);

	// allocate the MCL Image handle
	mImg = MclImgAlloc();
	// set the input file
	if (MclImgLoad (mImg,f,0)!= MCLERR_OK)
	{
		AfxMessageBox("Error in file loading");
		return 0;
	}

	// get information about the Image (width and height)
	MclImgGetInfo (mImg,&resX,&resY,bitsPix,totalBytes,pixPtr,palPtr);

	height = (int) resY;
	width = (int) resX;

	// alocate the RGB components
	allocateYUV();

	for (i=0; i < height; i++)
	{
		for (j=0; j<width ; j++)
		{
			MclImgGetPixelRGB (mImg,j,i,&r,&g,&b);
			Y[i][j] += (0.299 * r + 0.587 * g + 0.114 *b);
			U[i/2][j/2] += (-0.146*r - 0.288 * g + 0.434 *b)/4.0;
			V[i/2][j/2] += (0.617 *r - 0.517 * g - 0.1 *b)/4.0;
		}
	}

	// free the MCL handle
	MclImgFree(mImg);
	return 1;
}
//-------------------------------------------------------------------------
void CFrame::allocateYUV()
{
	int i,j;

	// Create the YUV arrays
	Y = (BYTE **) malloc(sizeof(BYTE*) * height);
	for (i=0; i < height; i++)
	{
		Y[i] = (BYTE*) malloc(sizeof(BYTE) * width);
	}
	U = (BYTE **) malloc(sizeof(BYTE*) * height/2);
	V = (BYTE **) malloc(sizeof(BYTE*) * height/2);
	for (i=0; i < height/2 ; i++)
	{
		U[i] = (BYTE*) malloc(sizeof(BYTE) * width/2);
		V[i] = (BYTE*) malloc(sizeof(BYTE) * width/2);
	}

	for (i = 0; i< height; i++)
		for (j=0; j < width; j++)
			Y[i][j] = U[i/2][j/2]=V[i/2][j/2]=0;
}
//------------------------------------------------------------------
void CFrame::calculateCH()
{
	CHcalculated = true;

	// USE Y, U, and V to calculate color histogram
	float temph[YCOLORRANGES][UCOLORRANGES][VCOLORRANGES];
	memset(temph,0,COLQUANT*4);
	for (int x = 0; x < height ;x++)
	{
		for (int y= 0; y < width ;y++)
		{
			temph[(int) Y[x][y]/(256/YCOLORRANGES)][(int) U[x/2][y/2]/(256/UCOLORRANGES)][(int) V[x/2][y/2]/(256/VCOLORRANGES)]++;
		}
	}

	memcpy(CH,temph,COLQUANT*4);
	int M = height * width;
	for (int j=0; j < COLQUANT; j++)
		CH[j] = CH[j]/(float) M;
}

//---------------------------------------------------------------------------
char * CFrame::getDC()
{
	// from the array of histogram values CH return the elements of the MPEG 7
	// descriptor of the dominant colors.
	char l[1000];

	if (!CHcalculated)
		calculateCH();
	
	strcpy(l,getDCFromCH(CH));
	strcat(l,"\n");
	return (l);
}

//----------------------------------------------------------------
char * getDCFromCH(float *ch)
{
	// this gets the dominant color in the array given in the argument
	// and return the set of dominant colors in the string
	int dc[COLQUANT],i,j;	// dc holds the indexes of the dominant colors
	float h[COLQUANT], CHDC[COLQUANT];

	memcpy(h,ch,COLQUANT*4);
	memset(CHDC,0,COLQUANT*4);

	for (i=0; i<COLQUANT; i++) 
		dc[i] = i;

	for (i =0; i<NOOFDOMINANTCOLORS; i++)
	{
		for (j=i+1; j< COLQUANT; j++)
		{
			if (h[i] < h[j])
			{
				// swap h[i], h[j]
				float x = h[i];
				h[i] = h[j];
				h[j] = x;

				x = dc[i];
				dc[i] = dc[j];
				dc[j] = x;
			}
		}
		CHDC[dc[i]] = h[i];
		/*		
		Originally the output was 8 dimensions with the YUV component values
		int y,u,v;	// the indices for YUV 
		y = (dc[i]/(UCOLORRANGES * VCOLORRANGES));
		u = (dc[i]%(UCOLORRANGES * VCOLORRANGES))/VCOLORRANGES;
		v = (dc[i]%(UCOLORRANGES * VCOLORRANGES))%VCOLORRANGES;

		if (!h[i])
			sprintf(temp, " 0 %d %d %d",y,u,v);
		else
			sprintf(temp, " %f %d %d %d",h[i],y,u,v);
		strcat(s,temp);
		*/
		// The output now will be in 32 dimensions like the color histogram
		// with the percentages of the dominant colors in the positions of 
		// the dominant colors and zeros else where
		// Now i is a dominant color
	}
	return (getStringFromArray(CHDC,COLQUANT));
}

//-----------------------------------------------------------------

char * CFrame::getSCol(float *h2)
{
	int k,p,q,x;
	float h[COLQUANT], H[COLQUANT], v1,v2,v3;
	double pp;
	char l[1000], temp[50];

	if (!CHcalculated)
		calculateCH();

	memset(H,0,COLQUANT*4);
	for (k=0; k < COLQUANT; k++)
	{
		if (k>0)
		{
			p = (float)log(k)/(float)log(2);;
			q = k - pow(2,p) + 1;
			pp = pow(2,p);
			v1 = (float)(COLQUANT*(q-1))/pp;
			v2 = (float) COLQUANT*((float)q-0.5)/pp;
			v3 = (float) (COLQUANT*q)/pp;
		}
		for (x = 0; x < COLQUANT; x++)
		{
			if (!k)
				h[x] = 1/sqrtf(COLQUANT);
			else
			{
				if (x >= v1 && x <= v2)
					h[x] = pow(2,((float)p/2.0))/sqrtf(COLQUANT);
				else if (x > v2 && x<= v3)
					h[x] = -1 * pow(2,((float)p/2.0))/sqrtf(COLQUANT);
				else
					h[x] = 0;
			}

			H[k] += (CH[x] * h[x]);
		}
		H[k]/= sqrtf(COLQUANT);
		
		h2[k] = H[k];
	}
	strcpy(l,getStringFromArray(H,COLQUANT));
	strcat(l,"\n");
	return l;
}

//--------------------------------------------------------------------
char * CFrame::getHT(float *h)
{
	const int HTENERGY_DIM = HT_DIM -2;

	int i,j,k;
	float avg, std, energy[HTENERGY_DIM], M, temp;
	char l[1000];

	memset(energy,0,4*HTENERGY_DIM);
	avg=std=0;
	M = height * width;

	for (i=0; i<height; i++)
		for (j=0; j<width;j++)
			avg+= Y[i][j];
	avg /= M;
	h[0] = avg;

	for (i=0; i<height; i++)
		for (j=0; j<width;j++)
			std += (Y[i][j] - avg)*(Y[i][j]-avg);
	std /= M;
	h[1] =std;

	sprintf(l," %f %f",avg,std);
	temp = 256.0 / 30.0;
	for (k=0; k<HTENERGY_DIM; k++)
	{
		for (i=0; i<height; i++)
			for (j=0; j<width;j++)
				energy[k] += (Y[i][j] - temp * (float) k) * (Y[i][j] - temp * (float) k);
			
		energy[k] /= M;
		h[k+2] = energy[k];
	}
	strcat(l,getStringFromArray(energy,HTENERGY_DIM));
	strcat(l,"\n");
	return l;
}

//----------------------------------------------------------------------
char * CFrame::getEH(float *h)
{
	// the image is devided to 16-subimage inside each subimage 4*4 blocks
	// we detect the direction of each block. The output is for each subimage
	// the frequency of blocks in each direction (80 dimension)

	int HE,VE,E45,E135,T;// values for each pixel
	int f[256], YE[MImage_H][MImage_W], directions[MImage_H][MImage_W];		// array for YE frequencies
	float binCounts[EH_DIM];
	int  i,j, m,n;
	float fDirection[5];	// directions inside each block
	char l[2000];
	int const blockW = 4;
	int const blockH = 4;

	memset(f,0,256*4);
	memset(binCounts,0,EH_DIM*4);
	
	// consider only insider pixels
	for (i=1; i<height-1; i++)
	{
		for (j=1; j<width; j++)
		{
			// Insider pixel
			VE = (Y[i-1][j-1] + 2*Y[i][j-1] + Y[i+1][j-1] - (Y[i-1][j+1] + 2*Y[i][j+1] + Y[i+1][j+1]));
			YE[i][j] = getAbsolute(VE);
			directions[i][j] = 0;		// vertical

			HE = (Y[i-1][j-1] + 2 * Y[i-1][j] + Y[i-1][j+1] - (Y[i+1][j-1] + 2*Y[i+1][j] + Y[i+1][j+1]));
			if (getAbsolute(HE) > YE[i][j])
			{
				YE[i][j] = getAbsolute(HE);
				directions[i][j] = 1; //horizontal
			}

			E45 = (2*Y[i-1][j-1] + Y[i-1][j] + Y[i][j-1] - (Y[i+1][j] + Y[i][j+1] + 2*Y[i+1][j+1]));
			if (getAbsolute(E45) > YE[i][j])
			{
				YE[i][j] = getAbsolute(E45);
				directions[i][j] = 2;		// 45 edge
			}
	
			E135 = (Y[i-1][j] + 2*Y[i-1][j+1] + Y[i][j+1] - (Y[i][j-1] + 2*Y[i+1][j-1] + Y[i+1][j]));
			if (getAbsolute(E135) > YE[i][j])
			{
				YE[i][j] = getAbsolute(E135);
				directions[i][j] = 3;	// 135 edge
			}

			if (YE[i][j] > 255)
				f[255] ++;
			else
				f[YE[i][j]]++;
		}
	}

	// all the frame pixels are processed
	T = getThreshold(f,256,(height * width)-1);

	for (i=1; i<height - 1; i ++)
	{
		for (j=1; j<width - 1; j++)
		{
			if (YE[i][j] < T)
				directions[i][j] = 4;	// non edge
		}
	}

	for(n=0; n <height/blockH; n++)
		for (m=0; m<width/blockW;m++)
		{
			memset(fDirection,0,5*4);
			for (i=0; i< blockH; i++)
				for (j=0; j<blockW; j++)
				{
					int x = n*blockH + i;
					int y = m*blockW + j;

					if (x == 0 || x == height-1 || y==0 || y==width - 1)
						continue;
					else
						fDirection[directions[x][y]] ++;
				}

				// get the block direction by specifying the max pixels that has this direction
				int blockDirection = 0;
				for (i=1; i<5; i++)
					if (fDirection[i] > fDirection[i-1])
						blockDirection = i;

				i = m * 4 * blockW / width;	//index of subimage in x direction
				j = n * 4 * blockH / height;	//index of subimage in y direction

				// map the block index and diection to the one dimension array
				binCounts[(j*4+i)*5 + blockDirection]++;
		}

	float M = height * width / (4*4*blockH*blockW);	// number of blocks per subimage

	strcpy(l,"");
	for (i=0; i< EH_DIM; i++)
		h[i] = binCounts[i]/M;

	strcpy(l,getStringFromArray(h,EH_DIM));
	strcat(l,"\n");
	return l;
}
