// RGB2HSV.cpp: implementation of the RGB2HSV class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RGB2HSV.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//------------------------------------------------------------
//	Xingquan Zhu, Last Modification, Mar. 29, 2001

//	Usage:
//	JLFPTR means "unsigned char" data type.
//	float *R_G_B2HSVHistogram(JLFPTR *r,JLFPTR *g,JLFPTR *b, int Number, int * Dimension)

//	return the HSV color histogram of the "r", "g", "b" color components, the dimension of the Color histogram will be indicated by 
//	variant "Dimension". "Number" means how many datas in any of the input.

//	float *R_G_B2HSVHistogram(JLFPTR *r,JLFPTR *g,JLFPTR *b, int Number, int * Dimension)

//	return the HSV color histogram of the "rgb" color components, the dimension of the Color histogram will be indicated by 
//	variant "Dimension". "Number" means how many datas in inputs.
//	
//
//----------------------------------------------------------



//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

RGB2HSV::RGB2HSV()
{
	m_fHSVhist=new float[HScale*SScale*VScale];
}
//----------------------------------------------------------------------------------------

RGB2HSV::~RGB2HSV()
{
	delete(m_fHSVhist);
}

//-------------------------------------------------------------------------
float * RGB2HSV::R_G_B2HSVHistogram(JLFPTR *r,JLFPTR *g,JLFPTR *b, int Number, int * Dimension)
{
	float * m_fHSVh,* m_fHSVs,* m_fHSVv;

	m_fHSVh=new float[Number];
	m_fHSVs=new float[Number];
	m_fHSVv=new float[Number];
	R_G_BToHSV(r,g,b,m_fHSVh,m_fHSVs,m_fHSVv,Number);
	QuantiveHSV(m_fHSVh,m_fHSVs,m_fHSVv,Number);
	HSVHist(m_fHSVh,m_fHSVs,m_fHSVv,Number,m_fHSVhist);
	Dimension[0]=HScale*SScale*VScale;
	delete(m_fHSVh);
	delete(m_fHSVs);
	delete(m_fHSVv);
	return(m_fHSVhist);
}
//----------------------------------------------------------------------------
float * RGB2HSV::RGB2HSVHistogram(JLFPTR *rgb, int Number, int *Dimension)
{
	float * m_fHSVh,* m_fHSVs,* m_fHSVv;

	m_fHSVh=new float[Number/3];
	m_fHSVs=new float[Number/3];
	m_fHSVv=new float[Number/3];
	RGBToHSV(rgb,m_fHSVh,m_fHSVs,m_fHSVv,Number);
	QuantiveHSV(m_fHSVh,m_fHSVs,m_fHSVv,Number/3);
	HSVHist(m_fHSVh,m_fHSVs,m_fHSVv,Number/3,m_fHSVhist);
	Dimension[0]=HScale*SScale*VScale;
	delete(m_fHSVh);
	delete(m_fHSVs);
	delete(m_fHSVv);
	return(m_fHSVhist);

}
//---------------------------------------------------------------------------
void RGB2HSV::R_G_BToHSV(JLFPTR *r,JLFPTR *g,JLFPTR *b,float *h,float *s,float *v,int Number)
{
JLFPTR jMax,jMin;
for(int i=0;i<Number;i++)
 { jMax=Findmax(r[i],g[i]);
   jMax=Findmax(jMax,b[i]);
   jMin=Findmin(r[i],g[i]);
   jMin=Findmin(jMin,b[i]);
   v[i]=jMax;
   if(v[i]==0)
     {s[i]=0.0;
      h[i]=0.0;
      continue;
      }
   else
     {s[i]=(v[i]-jMin)/v[i];
     }
   if(s[i]==0.0)
     {h[i]=0.0;
      continue;
     }
   if(v[i]==(float)r[i])
     { h[i]=60*(g[i]-b[i])/s[i]/v[i];
     }
   if(v[i]==(float)g[i])
     { h[i]=60*(2+(b[i]-r[i])/s[i]/v[i]);
     }
   if(v[i]==(float)b[i])
     { h[i]=60*(4+(r[i]-g[i])/s[i]/v[i]);
     }
   if(h[i]<0)
     { h[i]=h[i]+360;
     }
 }

}
//---------------------------------------------------------------------------
void RGB2HSV::RGBToHSV(JLFPTR *rgb,float *h,float *s,float *v,int Number)
{
JLFPTR jMax,jMin;
for(int i=0;i<Number;i=i+3)
 { jMax=Findmax(rgb[i],rgb[i+1]);
   jMax=Findmax(jMax,rgb[i+2]);
   jMin=Findmin(rgb[i],rgb[i+1]);
   jMin=Findmin(jMin,rgb[i+2]);
   v[i/3]=jMax;
   if(v[i/3]==0)
     {s[i/3]=0.0;
      h[i/3]=0.0;
      continue;
     }
   else
     {s[i/3]=(v[i/3]-jMin)/v[i/3];
     }
   if(s[i/3]==0.0)
     {h[i/3]=0.0;
      continue;
     }
   if(v[i/3]==(float)rgb[i])
     { h[i/3]=60*(rgb[i+1]-rgb[i+2])/s[i/3]/v[i/3];
     }
   if(v[i/3]==(float)rgb[i+1])
     { h[i/3]=60*(2+(rgb[i+2]-rgb[i])/s[i/3]/v[i/3]);
     }
   if(v[i/3]==(float)rgb[i+2])
     { h[i/3]=60*(4+(rgb[i]-rgb[i+1])/s[i/3]/v[i/3]);
     }
   if(h[i/3]<0)
     { h[i/3]=h[i/3]+360;
     }
   //i=i+3;
 }
}
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
void RGB2HSV::QuantiveHSV(float *h,float *s,float *v,int Number)
{
float jTempScaleh,jTempScalev;
float jTempScales;
jTempScaleh=(float)360/HScale;
jTempScales=(float)1.0/SScale;
jTempScalev=(float)256/VScale;
for(int i=0;i<Number;i++)
 {
    /*
    if(h[i]==0&&s[i]==0)
     {
       jTempScalev=256/LScale;
       for(int j=0;j<LScale;j++)
        {
          if(v[i]>j*jTempScalev&&(v[i]<=(j+1)*jTempScalev))
           {v[i]=j+1;
            break;
           }
         }

     }
     */

	//------------
	if(h[i]==360.0)
	{
	h[i]=HScale-1;
	}
	else
	{
	for(int j=0;j<HScale;j++)
	{
		if(h[i]>=j*jTempScaleh&&(h[i]<(j+1)*jTempScaleh))
		{
		h[i]=(float)j;
		break;		
		}
	}
	}
	//-----------------
	if(v[i]==256)
	{
	v[i]=VScale-1;
	}
	else
	{
    	for(int j=0;j<VScale;j++)
     	{
      	if(v[i]>=j*jTempScalev&&(v[i]<(j+1)*jTempScalev))
       	{v[i]=(float)j;
        break;
       	}
     	}
	}
	//---------------------------
	if(s[i]==4)
	{
	s[i]=SScale-1;
	}
	else
	{
    	for(int j=0;j<SScale;j++)
     	{
      	if(s[i]>=j*jTempScales&&(s[i]<(j+1)*jTempScales))
       	{s[i]=(float)j;
        break;
       	}
     	}
	}
 }
}
//---------------------------------------------------------------------------
void RGB2HSV::HSVHist(float *h,float *s,float *v,int Number,float *Hist)
{
int jTemph,jTemps,jTempv;
int jTempHist;
memset(Hist,0,HScale*SScale*VScale*4);
for(int i=0;i<Number;i++)
 {
  jTemps=0;
  jTempv=0;
  jTempHist=0;
  jTemps=(int)s[i];
  jTemph=(int)h[i];
  jTempv=(int)v[i];

  //if(h[i]!=0||s[i]!=0)
   {
    jTempHist=(jTemph<<2)+jTemps;
    jTempHist=(jTempHist<<2)+jTempv;
    Hist[jTempHist]++;
   }
  /*
  else
   {
    if(v[i]>0)
     {
      v[i]=v[i]-1;
     }
    Hist[288+(int)v[i]]++;
   }
  */
 }
for(i=0;i<HScale*SScale*VScale;i++)
 {
  Hist[i]=Hist[i]/Number;
 }
}
//---------------------------------------------------------------------------

