// Session.cpp: implementation of the CSession class.
//
//////////////////////////////////////////////////////////////////////

#include <time.h>
#include "stdafx.h"
#include "WinClient.h"
#include "Session.h"
#include "OutputDlg.h"
#include "CursorRecord.h"
#include <afxtempl.h>
#include "process.h"
#include "VideoDlg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CSession *g_session;
extern CArray <CursorRecord*, CursorRecord*> g_cursor;
extern int g_dispErrors;
extern FILE * g_pTempImageFile;
int g_dispImageH = 120;
int g_dispImageW = 150;
CString g_playerPath;
CString g_displayResponseTime;
double qResponseTime;
int g_useYUVFile;
int g_playerJob;

/////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSession::CSession()
{
	CFile f;
	CFileException e;
	CString serverName, portNumber,machineName;
	g_playerJob = PLAYERIDLE;

	if (!f.Open("config.txt", CFile::modeRead,&e))
	{
		AfxMessageBox("can't open Config file.. Connect to default port");
		serverName = "alkhwarizmi.cs.purdue.edu";
		portNumber = "6005";
		machineName = "miretteadmin";
		g_playerPath = "";
		g_pTempImageFile= fopen("c:\temp.dat","w+b");
		g_displayResponseTime = "1";
		g_useYUVFile = 1;
	}
	else
	{
		CArchive ar( &f, CArchive::load);
		ar.ReadString(serverName);
		ar.ReadString(portNumber);
		ar.ReadString(machineName);
		ar.ReadString(g_playerPath);
		CString fName;
		ar.ReadString(fName);
		g_pTempImageFile=fopen(fName,"w+b");
		ar.ReadString(g_displayResponseTime);
		CString temp;
		ar.ReadString(temp);
		g_useYUVFile = atoi((const char *) temp);
		ar.Close();
		f.Close();
	}
	
//	videoDlg = new CVideoDlg();

	predConnection = new PredConnection(serverName, atoi(portNumber), machineName
		, "password",0, 1);

	if (!predConnection->IsValid()) 
	{
		AfxMessageBox("WinClient:Did not establish connection to server");
		delete predConnection;
		exit(EXIT_FAILURE);
	}
	// initialize the thread work
	m_hEventStartRead = CreateEvent(NULL, FALSE, FALSE, NULL); // auto reset, initially reset
	m_hEventReadDone = CreateEvent(NULL, TRUE, TRUE, NULL); // manual reset, initially set
	m_hEventKillReadThread = CreateEvent(NULL, FALSE, FALSE, NULL); // auto reset, initially reset
	m_hEventReadThreadKilled = CreateEvent(NULL, FALSE, FALSE, NULL); // auto reset, initially reset

	m_readThreadInfo.m_hEventStartRead = m_hEventStartRead;
	m_readThreadInfo.m_hEventReadDone = m_hEventReadDone;
	m_readThreadInfo.m_hEventKillReadThread = m_hEventKillReadThread;
	m_readThreadInfo.m_hEventReadThreadKilled = m_hEventReadThreadKilled;

	m_pReadWorkerThread = NULL;
}

//----------------------------------------------------------
CSession::~CSession()
{
	if (predConnection)
		delete (predConnection);

//	if (videoDlg)
//		delete (videoDlg);
	fclose(g_pTempImageFile);

	// kill the worker thread
	DWORD dwExitCode;
	if (m_pReadWorkerThread != NULL && GetExitCodeThread(m_pReadWorkerThread->m_hThread, &dwExitCode) && dwExitCode == STILL_ACTIVE)
	{
		// Kill the worker thread by setting the "kill thread" event.
		SetEvent(m_hEventKillReadThread);
		SetEvent(m_hEventStartRead);
		WaitForSingleObject(m_hEventReadThreadKilled, INFINITE);
	}
}

//-------------------------------------------
int CSession::execQuery(CString sql)
{

	if ((sql.GetLength() + 6) >= XXX_INPUT_BUFFER_MAX)
	{
		AfxMessageBox("WinClient::Overflowing input buffer");
		exit(EXIT_FAILURE);
	}
	else
		sql = "SQL:" + sql;

    time_t start, finish;
	time (&start);
	char buf[XXX_INPUT_BUFFER_MAX];
	strcpy(buf,sql);
	strcat(buf,"\n");

	if (!predConnection->SendRequest(buf)) 
	{
		AfxMessageBox("WinClient:Error sending request to server");
		return 0;
		//delete predConnection;
		//exit(EXIT_FAILURE);
    }
	
	int status = predConnection->saveReply();
	time (&finish);
	qResponseTime += difftime(finish,start);
	if (g_dispErrors)
	{
		if (status != RESULTOK) 
		{ // MIRETTE
			switch (status)
			{
			case ERRINVALIDCON:
				AfxMessageBox("Invalid connection");
				break;
			case ERR0BYTE:
				AfxMessageBox("Server Error: Zero byte read");
				break;
			case ERRSYNTAX:
				AfxMessageBox("The problem may be that: \n - there is query syntax error;\n - or you executed a query that returns no result");
				break;
			case ERRINVALIDTYPE:
				AfxMessageBox("The query has a type in the result set that is not implemented");
				break;
			}
		}
    }

//	if (status <> RESULTOK)
//		return 0;
	
//	return 1;
	return status;
}
//------------------------------------------------
int CSession::drawResults(CDC *pDC, int &maxY, int &maxX)
{
	int imageX,imageY, textX,textY, lineSpace;
	CString content;
	void * img;

	imageY = textY = imageX = lineSpace =20;
	textX = 200;
	maxX = 0;	// the max string length
	HDC  hDC;  // a handle to a device context
	hDC = pDC->m_hDC;

	for (int i=0; i<g_cursor.GetSize();i++)
	{
		textY= imageY = i * ( 120 + 10 ) + 10;
		int j = 0;
		while ((g_cursor.GetAt(i))->fields[j].size != NULLSIZE)
		{
			if ((g_cursor.GetAt(i))->fields[j].type == IMAGE)
			{
				// display an Image
				DWORD  dwSize  = (g_cursor.GetAt(i))->fields[j].size;
				HGLOBAL hMemory = ::GlobalAlloc( GMEM_MOVEABLE, dwSize );
				LPVOID  pData = ::GlobalLock( hMemory );
				img = malloc(dwSize*sizeof(BYTE));
				long oldPos = ftell(g_pTempImageFile);
				long x = *((long *) (g_cursor.GetAt(i))->fields[j].content);
				fseek(g_pTempImageFile,x,SEEK_SET);
				fread(img,sizeof(BYTE),dwSize,g_pTempImageFile);
				fseek(g_pTempImageFile,oldPos,SEEK_SET);
				////::CopyMemory(pData,(g_cursor.GetAt(i))->fields[j].content,dwSize);
				::CopyMemory(pData,img,dwSize);				
				::GlobalUnlock( hMemory );
				free (img);
				IStream * pStream  = NULL;
				::CreateStreamOnHGlobal( hMemory, TRUE, &pStream );

				IPicture * pPicture = NULL;
				if( SUCCEEDED( ::OleLoadPicture( pStream, dwSize, TRUE, IID_IPicture,( LPVOID* )&pPicture ) ) )
				{ 
					OLE_XSIZE_HIMETRIC hmWidth;
					OLE_YSIZE_HIMETRIC hmHeight;
					pPicture->get_Width( &hmWidth );
					pPicture->get_Height( &hmHeight );
					pPicture->Render(hDC,imageX,imageY,g_dispImageW ,g_dispImageH ,0,hmHeight,hmWidth,-hmHeight,NULL);
					ULONG xx = pPicture->Release();
				}
				ULONG yy = pStream->Release();
				::GlobalFree(hMemory);
			}
			else if ((g_cursor.GetAt(i))->fields[j].type < IMAGE)
			{
				// display the data it's all strings
				CString s, s2;
				s = (const char*)((g_cursor.GetAt(i))->fields[j].fieldName);
				s += " : ";
				s2 = (const char *)((g_cursor.GetAt(i))->fields[j].content);
				s += s2;

				pDC->TextOut(textX,textY, s);
				textY+= lineSpace;
				
				if (s.GetLength() > maxX)
					maxX = s.GetLength();
			}
			j++;
		}
	}

	if (imageY + 120 > textY)
		maxY = imageY+120+10  ;
	else
		maxY = textY + 10;

	maxX = maxX*8 + textX;
	return 1;
}
//----------------------------------------------------
void CSession::clearResults()
{
	int crtArraySize = g_cursor.GetSize();
	for (int i=crtArraySize-1; i >= 0 ;i--)
	{
		if (g_cursor.GetAt(i))
			//delete (g_cursor.GetAt(i));
			delete(g_cursor.ElementAt(i));
	}

	g_cursor.RemoveAll();
	if (g_pTempImageFile)
		rewind(g_pTempImageFile);
}
//--------------------------------------------------------------------------
int CSession::execBatch(CString &fileName)
{
	CFile f;
	CFileException e;
	CString line, temp;

	if (!f.Open(fileName, CFile::modeRead,&e))
	{
		AfxMessageBox("can't open batch file");
		return 0;
	}

	CArchive ar( &f, CArchive::load);
	while (1)
	{
		ar.ReadString(line);
		if (line == "")
			break;

		temp = line.Left(2); 
		if (temp == "//")
			continue;

		temp = line.Left(3);
		temp.MakeUpper();
		if (temp == "REM")
			continue;

		execQuery(line);
	}

	ar.Close();
	f.Close();

	AfxMessageBox("Finish Batch Proccessing");
	return 1;
}
//-----------------------------------------------------------------------

int CSession::handleDoubleClick(CPoint point)
{
	// check if the double click is in a place that is working (on an image)
	if (point.x > 20 && point.x < g_dispImageW+20 && point.y > 10 && point.y < (g_dispImageH + 10) * g_cursor.GetSize())
	{
		int rowNo = (point.y - 10)/ (g_dispImageH + 10);
	
		// check if this image has the videoId and shot id selected with it
		// also check that there is an image in the current position
		BOOL videoIdOK, startFrameOK, endFrameOK, imageOK;
		videoIdOK = startFrameOK = endFrameOK = imageOK = FALSE;
		CString videoId, startFrame, endFrame;
		int j=0;

		while (!(videoIdOK && startFrameOK && endFrameOK && imageOK))
		{
			if (g_cursor.GetAt(rowNo)->fields[j].size == NULLSIZE)
				break;

			if (!strcmp(g_cursor.GetAt(rowNo)->fields[j].fieldName,"VideoId"))
			{
				videoIdOK = TRUE;
				videoId = (const char *) g_cursor.GetAt(rowNo)->fields[j].content;
			}
			else if (!strcmp(g_cursor.GetAt(rowNo)->fields[j].fieldName,"StartFrame"))
			{
				startFrameOK = TRUE;
				startFrame = (const char *) g_cursor.GetAt(rowNo)->fields[j].content;
			}
			else if (!strcmp(g_cursor.GetAt(rowNo)->fields[j].fieldName,"EndFrame"))
			{
				endFrameOK = TRUE;
				endFrame = (const char *) g_cursor.GetAt(rowNo)->fields[j].content;
			}
			else if (g_cursor.GetAt(rowNo)->fields[j].type == IMAGE)
				imageOK = TRUE;

			j++;
		}

		if (!imageOK)
			return 0;

		if (!(videoIdOK && startFrameOK && endFrameOK))
		{
			AfxMessageBox("This query should contain the video id, StartFrame, and EndFrame to display the video");
			return 0;
		}

	
		// If we are here then we have a video id, startFrame, endFrame and can find the results
		CString sql = "select v.Movie.videoURL("+startFrame+","+endFrame +") from VideoStream v where v.VideoId=" + videoId + ";";
		if (execQuery(sql) == RESULTOK)	// the result will be in the last element in g_cursor
		{
			// get the URL and delete the added element
			CString URL = (const char *) ((g_cursor.GetAt(g_cursor.GetUpperBound()))->fields[0].content);
			URL += "/vdbms.mpg";
			URL.Replace("\"","");
			//int x = _spawnl(_P_NOWAIT,g_playerPath,g_playerPath,URL,NULL);
//			videoDlg->URL = URL;
//			videoDlg->DoModal();
			
			delete (g_cursor.ElementAt(g_cursor.GetUpperBound()));
			g_cursor.RemoveAt(g_cursor.GetUpperBound(),1 );	// remove the URL
		}
		else
		{
			AfxMessageBox("Error Getting URL");
			return 0;
		}
	}
	return 1;
}

//---------------------------------------------------------------------------------

int CSession::drawResultsInDlg(CDC *pDC, int startIdx, int endIdx, int total)
{
	if (!total)
		return 0;

	int imageX,imageY;
	CString content;
	void * img;

	HDC  hDC;  // a handle to a device context
	hDC = pDC->m_hDC;
	
	CWnd * dlgW = pDC->GetWindow();
	int crtCtrlId = (dlgW->GetDlgItem(IDC_TEXT0))->GetDlgCtrlID();

	imageX = XDISPLAY;
	imageY = YDISPLAY;

	for (int i=startIdx; i<=endIdx;i++)
	{
		int j = 0;
		while ((g_cursor.GetAt(i))->fields[j].size != NULLSIZE)
		{
			if ((g_cursor.GetAt(i))->fields[j].type == IMAGE)
			{
				// display an Image
				DWORD  dwSize  = (g_cursor.GetAt(i))->fields[j].size;
				HGLOBAL hMemory = ::GlobalAlloc( GMEM_MOVEABLE, dwSize );
				LPVOID  pData = ::GlobalLock( hMemory );
				img = malloc(dwSize*sizeof(BYTE));
				long oldPos = ftell(g_pTempImageFile);
				long x = *((long *) (g_cursor.GetAt(i))->fields[j].content);
				fseek(g_pTempImageFile,x,SEEK_SET);
				fread(img,sizeof(BYTE),dwSize,g_pTempImageFile);
				fseek(g_pTempImageFile,oldPos,SEEK_SET);
				::CopyMemory(pData,img,dwSize);				
				::GlobalUnlock( hMemory );
				free (img);
				IStream * pStream  = NULL;
				::CreateStreamOnHGlobal( hMemory, TRUE, &pStream );

				IPicture * pPicture = NULL;
				if( SUCCEEDED( ::OleLoadPicture( pStream, dwSize, TRUE, IID_IPicture,( LPVOID* )&pPicture ) ) )
				{ 
					OLE_XSIZE_HIMETRIC hmWidth;
					OLE_YSIZE_HIMETRIC hmHeight;
					pPicture->get_Width( &hmWidth );
					pPicture->get_Height( &hmHeight );
					pPicture->Render(hDC,imageX,imageY,g_dispImageW ,g_dispImageH ,0,hmHeight,hmWidth,-hmHeight,NULL);
					ULONG xx = pPicture->Release();
				}
				ULONG yy = pStream->Release();
				::GlobalFree(hMemory);

				imageX += g_dispImageW;
				imageX += 10;
			}
			else if ((g_cursor.GetAt(i))->fields[j].type == STRINGURL)
			{
				if (g_playerJob != PLAYERSTREAMING)
				{
					CString URL = (const char *) ((g_cursor.GetAt(i))->fields[j].content);
					URL.Replace("\"@MP:","");
					URL.Replace("\"","");
					// pass the URL to the media player
					COutputDlg * outDlgW = (COutputDlg*)dlgW;
					outDlgW->m_player.Open(URL);
					g_playerJob = PLAYERSTREAMING;
				}
			}
			else if ((g_cursor.GetAt(i))->fields[j].type < IMAGE)
			{
				// display the data it's all strings
				CString s, s2;
				if (!j)
				{
					char temp[10];
					sprintf(temp,"%d/%d",i+1,total);
					dlgW->SetDlgItemText(crtCtrlId,(const char * ) temp);
				}

				s = (const char*)((g_cursor.GetAt(i))->fields[j].fieldName);
				s += " : ";
				s2 = (const char *)((g_cursor.GetAt(i))->fields[j].content);
				s += s2;

				dlgW->GetDlgItemText(crtCtrlId,s2);
				s2 += "\r\n";
				s2 += s;
				dlgW->SetDlgItemText(crtCtrlId,s2);
			}
			j++;
		}
		crtCtrlId = (dlgW->GetNextDlgGroupItem(dlgW->GetDlgItem(crtCtrlId),FALSE))->GetDlgCtrlID();	// get the next text box in the group
	}

	if (startIdx >= endIdx)
	{
		// should clear the text boxes
	}
	if (g_displayResponseTime == "1")
	{
		char temp[10];
		crtCtrlId = (dlgW->GetDlgItem(IDC_RESPONSETIME))->GetDlgCtrlID();
		sprintf (temp,"%g",qResponseTime);
		content = (const char *) temp;
		content = "Query Resp. Time " + content + " Sec"; 
		dlgW->SetDlgItemText(crtCtrlId,content);
	}
	return 1;
}
//----------------------------------------------------------------
int CSession::getResultsSize()
{
	return (g_cursor.GetSize());
}
//-----------------------------------------------------------------------
int CSession::handleDoubleClickInDlg(CPoint point, int startIdx, CString &URL)
{
	// check if the double click is in a place that is working (on an image)
	if (point.x>=XDISPLAY  && point.x <= XDISPLAY + (g_dispImageW + 10) * IMAGES_IN_DISPLAY && point.y >= YDISPLAY && point.y <= g_dispImageH + YDISPLAY)
	{
		int colNo = (point.x - XDISPLAY)/ (g_dispImageW + 10);
		int id = colNo + startIdx;
		
		// check if this image has the videoId and shot id selected with it
		// also check that there is an image in the current position
		BOOL videoIdOK, startFrameOK, endFrameOK, imageOK;
		videoIdOK = startFrameOK = endFrameOK = imageOK = FALSE;
		CString videoId, startFrame, endFrame;
		int j=0;

		while (!(videoIdOK && startFrameOK && endFrameOK && imageOK))
		{
			if (g_cursor.GetAt(id)->fields[j].size == NULLSIZE)
				break;

			if (!strcmp(g_cursor.GetAt(id)->fields[j].fieldName,"VideoId"))
			{
				videoIdOK = TRUE;
				videoId = (const char *) g_cursor.GetAt(id)->fields[j].content;
			}
			else if (!strcmp(g_cursor.GetAt(id)->fields[j].fieldName,"StartFrame"))
			{
				startFrameOK = TRUE;
				startFrame = (const char *) g_cursor.GetAt(id)->fields[j].content;
			}
			else if (!strcmp(g_cursor.GetAt(id)->fields[j].fieldName,"EndFrame"))
			{
				endFrameOK = TRUE;
				endFrame = (const char *) g_cursor.GetAt(id)->fields[j].content;
			}
			else if (g_cursor.GetAt(id)->fields[j].type == IMAGE)
				imageOK = TRUE;

			j++;
		}

		if (!imageOK)
			return 0;

		if (!(videoIdOK && startFrameOK && endFrameOK))
		{
			AfxMessageBox("This query should contain the video id, StartFrame, and EndFrame to display the video");
			return 0;
		}

	
		// If we are here then we have a video id, startFrame, endFrame and can find the results
		CString sql = "select v.Movie.videoURL("+startFrame+","+endFrame +") from VideoStream v where v.VideoId=" + videoId + ";";
		int status = execQuery(sql);
		if (status == RESULTOK || status == IGNORERESULT)	// the result will be in the last element in g_cursor
		{
			// get the URL and delete the added element
			URL = (const char *) ((g_cursor.GetAt(g_cursor.GetUpperBound()))->fields[0].content);
			URL.Replace("\"","");
			URL.Replace("@MP:","");
			delete (g_cursor.ElementAt(g_cursor.GetUpperBound()));
			g_cursor.RemoveAt(g_cursor.GetUpperBound(),1 );	// remove the URL
			g_playerJob = PLAYERPLAYING;
		}
		else
		{
			URL  = "";
			AfxMessageBox("Error Getting URL");
			return 0;
		}
	}
	return 1;
}
//--------------------------------------------------------------------------
int CSession::readBuffer2ndThread()
{
	if (m_pReadWorkerThread == NULL)
	{
		// Begin the worker thread.  It is ok to fill in the CThreadInfo
		// structure after the thread has started, because the thread
		// waits for the "start read" event before referring to the structure.
		m_pReadWorkerThread = AfxBeginThread(readThreadProc, &m_readThreadInfo);
	}

	m_readThreadInfo.predConn = predConnection;

	// The events are initially set or reset in the CreateEvent call;
	// but they may be left in an improperly initialized state if
	// a worker thread has been previously started and then prematurely
	// killed.  Set/reset the events to the proper initial state.
	// Set the "start read" event last, since it is the event the
	// triggers the starting of the worker thread reading.
	SetEvent(m_hEventReadDone);
	ResetEvent(m_hEventKillReadThread);
	ResetEvent(m_hEventReadThreadKilled);
	SetEvent(m_hEventStartRead);

/*	int status = predConnection->ignoreReply();
	if (g_dispErrors)
	{
		if (status != RESULTOK) 
		{ // MIRETTE
			switch (status)
			{
			case ERRINVALIDCON:
				AfxMessageBox("Invalid connection");
				break;
			case ERR0BYTE:
				AfxMessageBox("Server Error: Zero byte read");
				break;
			case ERRSYNTAX:
				AfxMessageBox("The problem may be that: \n - there is query syntax error;\n - or you executed a query that returns no result");
				break;
			case ERRINVALIDTYPE:
				AfxMessageBox("The query has a type in the result set that is not implemented");
				break;
			}
		}
    }

	return status;
	*/
	return RESULTOK;
}
//------------------------------------------------------------------------
/*
This is the same as execQuery except that it doesn't save the result of the 
query, it just send it to the server.
I needed it specially for the stream_stop command.
*/
int CSession::execCommadQuery(CString sql)
{
	if ((sql.GetLength() + 6) >= XXX_INPUT_BUFFER_MAX)
	{
		AfxMessageBox("WinClient::Overflowing input buffer");
		exit(EXIT_FAILURE);
	}
	else
		sql = "SQL:" + sql;

	char buf[XXX_INPUT_BUFFER_MAX];
	strcpy(buf,sql);
	strcat(buf,"\n");

	if (!predConnection->SendRequest(buf)) 
	{
		AfxMessageBox("WinClient:Error sending request to server");
		return 0;
    }
	return 1;
}
//-------------------------------------------------------------
void CSession::killReadBufferThread()
{
	// NOT USED FOR NOW
	// The worker thread periodically checks for the "kill recalc" event
	// during its execution of a recalculation.  When the worker thread
	// is not currently recalculating, it is waiting for a "start recalc"
	// event.  Therefore, in order to kill the thread that might be waiting
	// for the "start recalc" event, it is necessary to first set the
	// "start recalc" event.
	SetEvent(m_hEventKillReadThread);
	SetEvent(m_hEventStartRead);

	WaitForSingleObject(m_hEventReadThreadKilled, INFINITE);

	m_pReadWorkerThread = NULL;
}
//------------------------------------------------
void CSession::deleteStreamingResults()
{

}

int CSession::cleanAfterStreaming()
{
	// delete the streaming reslts from the cursor so they will not be streamed again
	int crtArraySize = g_cursor.GetSize();
	for (int i=crtArraySize-1; i >= 0 ;i--)
	{
		if (g_cursor.GetAt(i))
		{
			delete(g_cursor.ElementAt(i));
			g_cursor.RemoveAt(i,1);
		}

	}
	
	AfxGetApp()->BeginWaitCursor();
	WaitForSingleObject(m_hEventReadDone, INFINITE);
	AfxGetApp()->EndWaitCursor();
	return (getResultsSize());
}
