/*
  Multimedia Conversion Library, version 2
  External Definitions
  Copyright 1998-2000 Robert Tibljas and Zeljko Nikolic
*/

#ifndef  MCLDEF_H
#define  MCLDEF_H


#include  <windows.h>

#include  "basicdef.h"


#ifdef  __cplusplus
extern "C" {
#endif


// version identification
#define  MCL_VERSION_MAJOR   2
#define  MCL_VERSION_MIDDLE  1
#define  MCL_VERSION_MINOR   2
#define  MCL_VERSION         212
  
// filename separator
#define  MCL_PATH_SEP  ';'  // used for file list functions

// record sizes (in bytes)
#define  MCL_PALETTE_SIZE  768
  
// max. string lengths (including the null terminator)
#define  MCL_STRLEN_FILENAME  264
#define  MCL_STRLEN_INFO      128
#define  MCL_STRLEN_TIME      16

// result
#define  MCLERR_OK              0     // no errors
#define  MCLERR_INVALID_OUTPUT  1     // invalid output file name
#define  MCLERR_USER_ABORTED    2     // conversion interrupted by MclAbort
#define  MCLERR_EXCEPTION       999   // exception occured
#define  MCLERR_SYSTEM          1000  // error during the conversion

// status, used in progress callback
#define  MCL_STATUS_READY      0  // ready for a new conversion (finished with the previous)
#define  MCL_STATUS_PREPARING  1  // conversion initialization
#define  MCL_STATUS_COUNTING   2  // while counting colors for palette generation
#define  MCL_STATUS_SAVING     3  // while converting and saving

// error callback action
#define  MCL_ACTION_ABORT   1  // abort conversion, erase output files
#define  MCL_ACTION_FINISH  2  // stop the conversion, close output files
#define  MCL_ACTION_RETRY   3  // try reading the problematic input file again
#define  MCL_ACTION_SKIP    4  // skip the problematic input file

// media type for MclListAdd
#define  MCL_MEDIA_ALL    0   // all supported audio and video formats
#define  MCL_MEDIA_IMAGE  1   // image formats only (JPG, PNG, BMP, etc.)
#define  MCL_MEDIA_MOVIE  2   // movie formats only (AVI, MPG, HAV, etc.)
#define  MCL_MEDIA_SOUND  4   // sound formats only (WAV, MP2, MP3, etc.)
#define  MCL_MEDIA_VIDEO  8   // all formats that can contain pictures
#define  MCL_MEDIA_AUDIO  16  // all formats that can contain sound
#define  MCL_MEDIA_QUICK_CHECK  0x70000000  // check just the file name extension
#define  MCL_MEDIA_NO_CHECK     0x7FFFFFFF  // no checking at all

// text style (can be combined with the OR operator)
#define  MCL_TEXT_BOLD       1
#define  MCL_TEXT_ITALIC     2
#define  MCL_TEXT_UNDERLINE  4
#define  MCL_TEXT_STRIKEOUT  8

// text effect
#define  MCL_TEXT_SHADOW_NONE     0
#define  MCL_TEXT_SHADOW_CLASSIC  1
#define  MCL_TEXT_SHADOW_BORDER   2

// text colors
#define  MCL_COLOR_FORE    16777215  // white
#define  MCL_COLOR_SHADOW  0         // black
#define  MCL_COLOR_BACK    8421504   // dark gray

// RGB order
#define  MCL_ORDER_RGB  0
#define  MCL_ORDER_BGR  1

// red, green and blue offset within mImg pixels
#define  MCL_R  2
#define  MCL_G  1
#define  MCL_B  0
  
// Common AVI CODECs
#define  MCL_AVI_DIB   0x20424944  // Full frames, uncompressed
#define  MCL_AVI_CVID  0x64697663  // Cinepak by Supermac (default)
#define  MCL_AVI_IV32  0x32335649  // Intel Indeo Video R3.2
#define  MCL_AVI_MSVC  0x6376736D  // Microsoft Video 1
#define  MCL_AVI_MRLE  0x656C726D  // Run Length Encoding

// MPEG codecs
#define  MCL_MPG_1  1
#define  MCL_MPG_2  2

// HAV CODECs
#define  MCL_HAV_BEST  0  // maximum compression
#define  MCL_HAV_FAST  1  // very fast compression

// TIFF CODECs
#define  MCL_TIF_RAW       1      // uncompressed
#define  MCL_TIF_LZW       5      // Lempel-Ziv & Welch
#define  MCL_TIF_JPEG      7      // JPEG DCT
#define  MCL_TIF_PACKBITS  32773  // Macintosh RLE
#define  MCL_TIF_DEFLATE   32946  // Deflate - ZIP
  
// Bayer colorOrMono
#define  MCL_BAY_COLOR  0
#define  MCL_BAY_MONO   1
// Bayer colorOrder
#define  MCL_BAY_BGGR  0
#define  MCL_BAY_GRBG  1

  
// handles
typedef  Qint  Qmcl;      // data handle
typedef  Qint  QmclImg;   // image handle
typedef  Qint  QmclSnd;   // sound handle
typedef  Qint  QmclText;  // text style handle
typedef  Qint  QmclList;  // file list handle
typedef  Qint  Qresult;   // error result

// error record
typedef struct {
  Qint  mclErrCode;
  Qint  errGroup;
  Qint  errCode;
} QmclError;

// fixed-length strings
typedef  Qchar  QstrFile[MCL_STRLEN_FILENAME];
typedef  Qchar  QstrInfo[MCL_STRLEN_INFO];

// file information
typedef struct {
  // file
  QstrFile  fName;     // file name
  QstrInfo  fFmtName;  // file format name (e.g. "AVI, Windows Audio Video Interleave")
  Qfloat    fSize;     // disk file size in bytes (limited to 2 GB for now)
  // video
  Qbool     vExists;     // file contains a video stream?
  Qbool     vSupported;  // is it supported by the library?
  QstrInfo  vFmtName;    // video format name (e.g. "(cvid) Cinepak Codec by Radius")
  Qint      vLen;        // length in frames
  Qfloat    vTimeLen;    // duration in seconds
  Qfloat    vBytesLen;   // size of the uncompressed video in bytes
  Qfloat    vFreq;       // frame rate in frames/second (FPS)
  Qint      resX;        // video width in pixels
  Qint      resY;        // video height in pixels
  Qint      resB;        // number of bytes per pixel
  Qint      bpp;         // number of bits per pixel (not always resB * 8 !!)
  // audio
  Qbool     aExists;      // file contains a audio stream?
  Qbool     aSupported;   // is it supported by the library?
  QstrInfo  aFmtName;     // audio format name (e.g. "(#1) PCM, uncompressed")
  Qint      aLen;         // length in samples
  Qfloat    aTimeLen;     // duration in seconds
  Qfloat    aBytesLen;    // size of the uncompressed audio in bytes
  Qfloat    aFreq;        // sample rate in samples/second
  Qint      aChannels;    // number of channels (1=mono, 2=stereo)
  Qint      aBitsChn;     // number of bits per channel (usually 8 or 16)
  Qint      aSampleSize;  // number of bytes per sample
} QmclFileInfo;

// codec information
typedef struct {
  Qint  fileFmtID;  // file format ID (see FileFmtDef.h)
  // codec
  Qbool     codecUsed;  // TRUE (1) if following codec* fields are valid
  Qint      codecID;    // compression/decompression ID number, for AVI this is a fourCC code (i.e. "CVID", "MSVC", etc.)
  Qint      codecMin;   // unused
  Qint      codecMax;   // unused
  QstrInfo  codecName;  // string containing codec name (i.e. "Cinepak codec by Radius")
  // quality
  Qbool  qualityUsed;  // TRUE (1) if following quality* fields are valid
  Qint   quality;      // default codec quality
  Qint   qualityMin;   // minimum value (usually 0%)
  Qint   qualityMax;   // maximum value (usually 100%)
  // data rate
  Qbool  dataRateUsed;  // TRUE (1) if following dataRate* fields are valid
  Qint   dataRate;      // default codec's data rate in kBps
  Qint   dataRateMin;   // minimum allowed data rate in kBps
  Qint   dataRateMax;   // maximum allowed data rate in kBps
  // key frame rate
  Qbool  keyFrameRateUsed;  // TRUE (1) if following keyFrameRate* fields are valid
  Qint   keyFrameRate;      // default value, key frames are frames compressed using only their own data and are inserted every keyFrameRate frames
  Qint   keyFrameRateMin;   // minimum allowed value
  Qint   keyFrameRateMax;   // maximum allowed value
} QmclCodecsInfo;

// MclImgRemap table
typedef  Qbyte           QmclRemapTable[256];
typedef  QmclRemapTable  *QmclRemapTableP;

// callback function prototypes
typedef  void  (__stdcall  *QmclSrcAddCBF) (Qmcl mData, Qint blockNum);
typedef  void  (__stdcall  *QmclVideoFuncCBF) (Qmcl mData, QmclImg mImg, Qint curFrame);
typedef  void  (__stdcall  *QmclAudioFuncCBF) (Qmcl mData, QmclSnd mSnd, Qint curSample);
typedef  void  (__stdcall  *QmclProgressCBF) (Qmcl mData, Qint curState, Qint doneFrames, Qint doneSamples);
typedef  void  (__stdcall  *QmclErrorCBF) (Qmcl mData, QcharP inputFile, Qint *action);


#define  MCLFN(type,name)  extern type __stdcall name


MCLFN (Qbool, MclRegister) (QcharP regCode);
MCLFN (void, MclEnableMPEG2) (void);
MCLFN (void, MclEnableLZW) (void);

MCLFN (Qresult, MclQuickConvert) (QcharP outputFile, QcharP inputVideo, QcharP inputAudio,
           Qint resX, Qint resY, Qfloat fps);

MCLFN (Qmcl, MclAlloc) (void);
MCLFN (void, MclFree) (Qmcl mData);
MCLFN (Qresult, MclConvert) (Qmcl mData);
MCLFN (void, MclAbort) (Qmcl mData);

MCLFN (void, MclGetFileInfo) (QcharP filename, QmclFileInfo *info,
           Qbool seqDetect);

MCLFN (void, MclSetOutputFile) (Qmcl mData, QcharP videoFile, QcharP audioFile);
MCLFN (void, MclSetFrameRate) (Qmcl mData, Qfloat fps, Qbool keepDuration);
MCLFN (void, MclSetResolution) (Qmcl mData, Qint resX, Qint resY);
MCLFN (void, MclSetBpp) (Qmcl mData, Qint bpp);
MCLFN (void, MclEnableDithering) (Qmcl mData, Qbool turnOn);
MCLFN (void, MclForceCompatibility) (Qmcl mData, Qbool turnOn);
MCLFN (void, MclSetAudioFormat) (Qmcl mData, Qint sampleRate, Qint channels, Qint bits);

MCLFN (Qint, MclCodecsInit) (Qint fileFmtID, Qint resX, Qint resY, Qint bpp);
MCLFN (void, MclCodecsGetInfo) (Qint fileFmtID, QmclCodecsInfo *info, Qint codecNum);

MCLFN (Qint, MclGetFileFmtID) (QcharP filename);
MCLFN (void, MclSetCodec) (Qmcl mData, Qint fileFmtID, Qint codec);
MCLFN (Qbool, MclCodecDlg) (Qmcl mData, Qint fileFmtID, Qint resX, Qint resY, Qint bpp);
MCLFN (void, MclSetQuality) (Qmcl mData, Qint fileFmtID, Qint value);
MCLFN (void, MclSetDataRate) (Qmcl mData, Qint fileFmtID, Qint kBps);
MCLFN (void, MclSetKeyFrameRate) (Qmcl mData, Qint fileFmtID, Qint keyFrameEvery);
MCLFN (Qbool, MclOutputDlg) (Qmcl mData, QmclList mList, QcharP title,
            Qbool enableChanges);

MCLFN (void, MclAssignVideoSource) (Qmcl mData, QmclSrcAddCBF funcname);
MCLFN (void, MclAssignVideoFunc) (Qmcl mData, QmclVideoFuncCBF funcname);
MCLFN (void, MclAssignAudioSource) (Qmcl mData, QmclSrcAddCBF funcname);
MCLFN (void, MclAssignAudioFunc) (Qmcl mData, QmclAudioFuncCBF funcName);
MCLFN (void, MclAssignProgress) (Qmcl mData, QmclProgressCBF funcname);
MCLFN (void, MclAssignError) (Qmcl mData, QmclErrorCBF funcname);

MCLFN (void, MclSrcSetFile) (Qmcl mData, QcharP filename);
MCLFN (void, MclSrcSetImg) (Qmcl mData, QmclImg mImg);
MCLFN (void, MclSrcSetDIB) (Qmcl mData, HBITMAP DIBhandle);
MCLFN (void, MclSrcSetPixels) (Qmcl mData, void *pixPtr, void *palPtr, Qint resX, Qint resY, Qint bpp);
MCLFN (void, MclSrcSetFrameRate) (Qmcl mData, Qfloat fps);
MCLFN (void, MclSrcSetDirection) (Qmcl mData, Qint direction);
MCLFN (void, MclSrcSetStart) (Qmcl mData, Qint quant);
MCLFN (void, MclSrcSetLen) (Qmcl mData, Qint quants);
MCLFN (void, MclSrcSetStartTime) (Qmcl mData, Qfloat sec);
MCLFN (void, MclSrcSetLenTime) (Qmcl mData, Qfloat sec);
MCLFN (void, MclSrcFinished) (Qmcl mData);

MCLFN (void, MclImgGamma) (QmclImg mImg, Qfloat gammaR, Qfloat gammaG, Qfloat gammaB);
MCLFN (void, MclImgBrightness) (QmclImg mImg, Qint pctR, Qint pctG, Qint pctB);
MCLFN (void, MclImgContrast) (QmclImg mImg, Qint pctR, Qint pctG, Qint pctB);
MCLFN (void, MclImgSoften) (QmclImg mImg);
MCLFN (void, MclImgSharpen) (QmclImg mImg);
MCLFN (void, MclImgEdgeEnhance) (QmclImg mImg);
MCLFN (void, MclImgResize) (QmclImg mImg, Qint resX, Qint resY);
MCLFN (void, MclImgResample) (QmclImg mImg, Qint resX, Qint resY);
MCLFN (void, MclImgBorder) (QmclImg mImg, Qint left, Qint top, Qint right, Qint bottom, Qint color);
MCLFN (void, MclImgRemap) (QmclImg mImg, QmclRemapTableP redP, QmclRemapTableP greenP, QmclRemapTableP blueP);
MCLFN (void, MclImgSkip) (QmclImg mImg);

MCLFN (QmclImg, MclImgAlloc) (void);
MCLFN (void, MclImgFree) (QmclImg mImg);
MCLFN (Qresult, MclImgLoad) (QmclImg mImg, QcharP filename, Qint frameNum);
MCLFN (Qresult, MclImgSave) (QmclImg mImg, QcharP filename);
MCLFN (void, MclImgCopy) (QmclImg mImgDest, QmclImg mImgSrc);
MCLFN (void, MclClearCanvas) (Qcard dc, Qint x, Qint y, Qint w, Qint h, Qint color);
MCLFN (void, MclImgShow) (QmclImg mImg, Qcard dc, Qint x, Qint y);
MCLFN (void, MclImgShowInRect) (QmclImg mImg, Qcard dc, Qint x, Qint y, Qint w, Qint h);
MCLFN (void, MclImgGetInfo) (QmclImg mImg, Qint *resX, Qint *resY, Qint *bpp, Qint *totalBytes,
        void **pixPtr, void **palPtr);
MCLFN (void, MclImgGetPixels) (QmclImg mImg, void *pixPtr, void *palPtr);
MCLFN (void, MclImgSetPixels) (QmclImg mImg, Qint resX, Qint resY, Qint bpp, void *pixPtr, void *palPtr);
MCLFN (void, MclImgGetPixelRGB) (QmclImg mImg, Qint x, Qint y, Qint *r, Qint *g, Qint *b);
MCLFN (HBITMAP, MclImgToDIB) (QmclImg mImg);
MCLFN (Qbool, MclReleaseDIB) (HBITMAP DIBhandle);
MCLFN (HANDLE, MclImgToDIBWin3) (QmclImg mImg);
MCLFN (Qbool, MclReleaseDIBWin3) (HBITMAP DIBhandle);
MCLFN (void, MclDIBToImg) (QmclImg mImg, HBITMAP DIBhandle);
MCLFN (Qresult, MclBayerLoad) (QmclImg mImg, QcharP filename, Qint resX, Qint resY,
        Qint colorOrMono, Qint colorOrder);
MCLFN (void, MclBayerToImg) (QmclImg mImg, void *pixPtr, Qint resX, Qint resY, 
           Qint colorOrMono, Qint colorOrder);

MCLFN (QmclSnd, MclSndAlloc) (void);
MCLFN (void, MclSndFree) (QmclSnd mSnd);
MCLFN (void, MclSndGetInfo) (QmclSnd mSnd, Qint *sampleRate, Qint *channels, Qint *bits, 
        Qint *totalBytes, void **sndPtr);
MCLFN (void, MclSndGetSamples) (QmclSnd mSnd, void *sndPtr);
MCLFN (void, MclSndSetSamples) (QmclSnd mSnd, Qint sampleRate, Qint channels, Qint bits, void *sndPtr);

MCLFN (QmclText, MclTextAlloc) (void);
MCLFN (void, MclTextFree) (QmclText mText);
MCLFN (void, MclTextSetFont) (QmclText mText, QcharP name, Qint size, Qint style);
MCLFN (void, MclTextSetColor) (QmclText mText, Qint foreColor, Qint shadowColor, Qint backColor,
           Qbool transp);
MCLFN (void, MclTextShadow) (QmclText mText, Qint shadowMode, Qint Xoffset, Qint Yoffset);
MCLFN (void, MclTextGetExtents) (QmclText mText, QcharP textStr,
           Qint *resX, Qint *resY);
MCLFN (void, MclTextOut) (QmclImg mImg, QmclText mText, Qint x, Qint y,
           QcharP textStr);

MCLFN (QmclList, MclListAlloc) (void);
MCLFN (void, MclListFree) (QmclList mList);
MCLFN (Qint, MclListAdd) (QmclList mList, QcharP fileSpec, Qint fileType);
MCLFN (void, MclListGetName) (QmclList mList, Qint index, QcharP filename);
MCLFN (Qint, MclListGetFileCount) (QmclList mList);
MCLFN (Qint, MclListGetFrameCount) (QmclList mList);
MCLFN (void, MclListGetMaxRes) (QmclList mList, Qint *resX, Qint *resY, Qint *bpp);
MCLFN (void, MclListGetAudioFormat) (QmclList mList, Qint *sampleRate, Qint *channels, Qint *bits);

MCLFN (Qfloat, MclGetElapsedTime) (Qmcl mData);
MCLFN (void, MclMakeTimeStr) (Qfloat seconds, QcharP timeStr);
MCLFN (Qint, MclGetStatus) (Qmcl mData);
MCLFN (void, MclGetLastError) (QmclError *mError);
MCLFN (void, MclMakeErrorStr) (QmclError *mError, QcharP errStr);
MCLFN (void, MclCheckError) (void);

MCLFN (Qint, MclTiffGetTag) (QcharP filename, Qint tagCode,
           void *bufferPtr, Qint bufferSize);
MCLFN (Qint, MclBayerGetHdr) (QcharP filename, void *bufferPtr,
           Qint bufferSize);

MCLFN (Qint, MclMakeFourCC) (QcharP fourCharCode);
MCLFN (void, MclPixelReorder) (void *inData, Qint inBpp, Qint inOrg,
           void *outData, Qint numPixels);
MCLFN (Qint, MclRGB) (Qint red, Qint green, Qint blue);

MCLFN (void, MclLogStart) (QcharP filename, Qbool extraInfo);
MCLFN (void, MclLogStop) (void);


#undef  MCLFN


#ifdef __cplusplus
}
#endif

#endif  // MCLDEF_H
