#include "stdafx.h"
#include "readthread.h"

UINT readThreadProc(LPVOID pParam)
{
	CReadThreadInfo* pReadInfo = (CReadThreadInfo*)pParam;

	// This read thread runs in an infinite loop, waiting to recalculate the
	// result whenever the main application thread sets the "start recalc" event.
	// The recalc thread exits the loop only when the main application sets the
	// "kill read" event.

	BOOL bReadCompleted;
	while (TRUE)
	{
		bReadCompleted = FALSE;

		// Wait until the main application thread asks this thread to do
		//      another read
		if (WaitForSingleObject(pReadInfo->m_hEventStartRead, INFINITE) != WAIT_OBJECT_0)
			break;

		// Exit the thread if the main application sets the "kill read"
		// event. The main application will set the "start read" event
		// before setting the "kill read" event.

		if (WaitForSingleObject(pReadInfo->m_hEventKillReadThread, 0) == WAIT_OBJECT_0)
			break; // Terminate this thread by existing the proc.

		// Reset event to indicate "not done", that is, recalculation is in progress.
		ResetEvent(pReadInfo->m_hEventReadDone);

		// Mirette: We don't need to check for the kill inside the ignoreReply 
		// function because we will not allow it. The function will exit when the
		// buffer is empty and it stopped by itself.
		int status = pReadInfo->predConn->ignoreReply();
		bReadCompleted = TRUE;
		
		// Set event to indicate that read is done (i.e., no longer in progres),
		// even if perhaps interrupted by "kill read" event detected in the SlowAdd function.
		SetEvent(pReadInfo->m_hEventReadDone);
	}

	if (!bReadCompleted)
		SetEvent(pReadInfo->m_hEventReadThreadKilled);

	return 0;
}
