#include "stdafx.h"
#include <ctype.h>
#include <iostream.h>
#include <string.h>
#include "socket.h"

#ifdef XXX_WIN32
#include <strstrea.h>
#else
#include <strstream.h>
#endif

#ifdef XXX_WIN32
#define strcasecmp(X, Y) _stricmp(X, Y)
#include <io.h>
#define write(A, B, C) _write(A, B, C)
#define read(A, B, C) _read(A, B, C)
#endif



/**
 ** ReplyScan class enables the user to iterate over the results returned
 ** from the server both line-wise and element within a line-wise.  This
 ** matrix-like parsing of the results produces clean and simple tabular
 ** representations of the results from queries sent to the server.
 ** HOWEVER!  Change the protocol to binary from textual and you instantly
 ** make the code better.  This is top priority.
 **/
class ReplyScan {
  // pointer to current position in the results buffer, set in init()
  char* ptr;
  // true if there are no results left in the result buffer, false otherwise
  bool atEnd;
public:
  // Stream which holds the results buffer from the server
  // aids heavily in interpretation of results buffer
  ostrstream AnsOStr;
  ReplyScan();
  ~ReplyScan();
  void init();
  // true if anything has been printed off the stream, false otherwise
  bool PrintedAlready;
  // True if there is an error returned from the server, false otherwise
  bool IsError();
  // returns the next character in the results buffer
  char GetNextChar();
  // 0 if there are no more results to parse, 1 if there are more results to parse
  // and it puts the current result into first, and increments the results pointer
  int GetNextItem(char* first);
  // True if there are any results left in the result buffer, false otherwise
  bool AnyResults();
  // feeds the next result line into OStr
  void GetNextLine(ostream& OStr);
  // feeds the next result line into OStr formatted for an HTML table
  void GetNextLineFormatted(ostream& OStr);
  // prints out the schema in formatted html tabular form
  void PrintSchema(ostream& OStr);
  // true if the result buffer is at the end, false otherwise
  bool AtEOF() const { return (atEnd ||(!*ptr));}
};

/**
 ** PredConnection class encapsulates all functionality necessary for dialog
 ** with a PREDATOR server by opening a connection to allow queries to be sent
 ** to the server and results received easily
 **/
#define RESULT_BUFFER_MAX 4096

class PredConnection {
 
  static const char *initString;

  int portNumber;
  char* hostName;
  char* userName;
  char* password;
  XxxSocketConnection* socket;
  bool binaryMode;
  bool getSchemas;

  bool valid;
  

  //// nullify all pointers
  void init() {
    valid = false;
    hostName = NULL;
    userName = NULL;
    password = NULL;
    socket = NULL;
    binaryMode = false;
    getSchemas = true;
   }
  //// reclaim all allocated pointers
  void reclaim() {
    if (hostName) delete [] hostName;
    if (userName) delete [] userName;
    if (password) delete [] password;
    if (socket) delete socket;
   }
  
  //// returns 0 if protocol setup fails, else returns 1
  int setupProtocol();
  //// returns true if no more data is expected from the server
  bool requestTerminated(const char* RecvdData, int BytesRead) const;
  //// returns 0 if protocol temination fails, else returns 1
  int terminateProtocol();
 public:
	 int ignoreReply();
	 int getSize(char * &crtPtr);
	 int saveReply();

  PredConnection(const char* HostName, int PortNumber,
		 const char* UserName, const char* Password,
		 bool Binary = false, bool GetSchemas = false);
  ~PredConnection();
  bool IsValid() const { return valid;}
  //// return 0 if error occurs, else returns 1
  int SendRequest(const char* Buf);
  //// return 0 if error occurs, else returns 1
  int PrintReply(ostream& OStr);
  //// returns a ReplyScan object which holds the response from the server
  ReplyScan* CreateReplyScan();
  //// returns a Buffer object which holds part of the response from the server
  bool CreateBufferScan(char* Buffer, bool &TimeOut, bool& AtEnd);
 };


