#include "stdafx.h"
#include "socket.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#if defined(__GNUG__) && defined(solaris)
extern "C" {
     extern int shutdown(int, int);
     extern int socket(int, int, int);
 };
#endif

/*
 * Read "n" bytes from a socket descriptor.
 */

int XxxSocketConnection::ReadN(char* ptr, int nbytes)
{
 DWORD   nleft, nread;
 
 if ((socketDesc == -1)|| (socketType != XXX_TALKSOCK)) return -1;

 nleft = nbytes;
 while (nleft > 0) {
   nread = recv(socketDesc,ptr,nleft,0);
   if (nread < 0) {
     perror("readn");
     return(nread);		/* error, return < 0 */
    }
   else if (nread == 0)
     break;			/* EOF */
    
   nleft -= nread;
   ptr   += nread;
  }
 return(nbytes - nleft);		/* return >= 0 */
}

/*
 * Write "n" bytes to a socket descriptor.
 */

int XxxSocketConnection::WriteN(char* ptr, int nbytes)
{
  // TODO: check this ... 
 DWORD   nleft, nwritten;

 if ((socketDesc == -1)|| (socketType != XXX_TALKSOCK)) return -1;

/** return send(socketDesc, (void *)ptr, nbytes, 0); **/

 nleft = nbytes;
 
 while (nleft > 0) {
   nwritten = send(socketDesc, ptr, nleft, 0);
   if (nwritten <= 0) {
     perror("writen");
     return(nwritten);		// error
    }
    
   nleft -= nwritten;
   ptr   += nwritten;
  }
 return(nbytes - nleft);
 
}

/*
 * Read a line from a socket descriptor.  Read the line one byte at a time,
 * looking for the newline.  We store the newline in the buffer,
 * then follow it with a null (the same as fgets(3)).
 * We return the number of characters up to, but not including,
 * the null (the same as strlen(3)).
 */
int XxxSocketConnection::ReadLine(char* ptr, int maxlen)
{
 int	n;
 DWORD  rc;
 char	c;
 
 if ((socketDesc == -1)|| (socketType != XXX_TALKSOCK)) return -1;

 for (n = 1; n < maxlen; n++) {
   if ( (rc = recv(socketDesc,&c,1,0)) == 1) {
     *ptr++ = c;
     if (c == '\n')
       break;
    } else if (rc == 0) {
      if (n == 1)
	return(0);		/* EOF, no data read */
      else
	break;			/* EOF, some data was read */
     } else {
       perror("readnetline");
       return(-1);		/* error */
      }
  }
 
 *ptr = 0;
 return(n);
}


/*
 * Open a TCP socket (an Internet stream socket) as a server
 */

XxxSocketConnection::XxxSocketConnection(int Port)
{
 struct sockaddr_in	serv_addr;
 
 while ( (socketDesc = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
   socketDesc = -1;
   return;
 }

 /*
  * make sure that local addresses can be reused
  */
 int ReUseAddr = 1;
 if (setsockopt(socketDesc, SOL_SOCKET, SO_REUSEADDR,
		(char *)&ReUseAddr, sizeof(ReUseAddr)) < 0) {
   fprintf(stderr,"unable to setsockopt REUSEADDR\n");
 }
 
 /*
  * Bind the local address so that the client can send to this socket.
  */
 //bzero((char *) &serv_addr, sizeof(serv_addr));
 memset((void *) &serv_addr, 0, sizeof(serv_addr));
 serv_addr.sin_family      = AF_INET;
 serv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
 serv_addr.sin_port        = htons(Port);
 
 if (bind(socketDesc, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0) {
   socketDesc = -1;
   return;
 }
 listen(socketDesc, 5);

 socketType = XXX_SERVSOCK;
}


XxxSocketConnection::XxxSocketConnection(int SockDesc, int SockType)
{
 socketDesc = SockDesc;
 socketType = SockType;
}

XxxSocketConnection::XxxSocketConnection(char *HostName, int Port)
{
 struct sockaddr_in	serv_addr;
 struct hostent *host;

 /*
  * Fill in the structure "serv_addr" with the address of the
  * server that we want to connect with.
  */
 if(!HostName || strlen(HostName) == 0 ) {
   HostName = "localhost";
 }
  
 host = gethostbyname(HostName);
 if (!host) {
   fprintf(stderr,"client: unknown host name");
   socketDesc = -1;
   return;
 }
  
 serv_addr.sin_family      = AF_INET;
 serv_addr.sin_addr.s_addr = ((struct in_addr*)(host->h_addr))->s_addr;
 serv_addr.sin_port        = htons(Port);

 if ((socketDesc = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
   fprintf(stderr,"client: can't open stream socket\n");
   socketDesc = -1;
   return;
  }
 if (connect(socketDesc, (struct sockaddr*)&serv_addr,sizeof(serv_addr)) < 0){
   fprintf(stderr,"client: can't connect to server");
   socketDesc = -1;
   return;
  }
 socketType = XXX_TALKSOCK;
}

XxxSocketConnection *XxxSocketConnection::AcceptConnection()
{
 if ((socketDesc == -1) || (socketType != XXX_SERVSOCK))
   return NULL;

 struct sockaddr Addr;
 int AddrLen = sizeof(Addr);
 
 int SockDesc;
 while ((SockDesc = accept(socketDesc, &Addr, &AddrLen)) == -1) {
#ifndef XXX_WIN32 // SCK - Multithreaded version do not support errno
   if (errno == EINTR) continue; // handle signals that interrupt socket
#endif
   perror("SockDesc");
   return NULL;
  }
 return new XxxSocketConnection(SockDesc, XXX_TALKSOCK);
}

XxxSocketConnection::~XxxSocketConnection()
{ 
 if (socketDesc != -1) {
#ifdef XXX_WIN32
   // Do a graceful shutdown: first make sure that data is flushed.
   // This is important for callbacks, otherwise wierd stuff happens
   if (WSAIoctl(socketDesc, SIO_FLUSH, 0, 0, 0, 0, 0, 0, 0) != 0) {
     // fprintf(stderr, "Error in WSAIoctl\n");
     // Don't bother printing it: there is always an error at shutdown
   }
   shutdown(socketDesc, SD_SEND);
   closesocket(socketDesc);
#else
   shutdown(socketDesc, 2);
   close(socketDesc);
#endif
  }
}

XxxSocketStarter::XxxSocketStarter()
{
#ifdef XXX_WIN32
 //initialize the socket library
 WORD wVersionRequested;
 WSADATA wsaData;
 int err;
 
 wVersionRequested = MAKEWORD( 2, 0 );
 
 err = WSAStartup( wVersionRequested, &wsaData );
 if ( err != 0 ) {
   fprintf(stderr, "Failed to initialize Winsock library\n");
   }
#endif
}

XxxSocketStarter::~XxxSocketStarter()
{
#ifdef XXX_WIN32
 int err = WSACleanup();
 if ( err != 0 ) {
   fprintf(stderr, "Failed to close Winsock library\n");
 }
#endif
}

